import Node from "./Node";
import RowNode from "./RowNode";
import Action from "./Action";
import TabSetNode from "./TabSetNode";
import BorderSet from "./BorderSet";
import DropInfo from "../DropInfo";
import Rect from "../Rect";
import IDraggable from "./IDraggable";
/**
 * Class containing the Tree of Nodes used by the FlexLayout component
 */
declare class Model {
    private static _attributeDefinitions;
    private _attributes;
    private _idMap;
    private _nextId;
    private _changeListener?;
    private _root?;
    private _borders;
    private _onAllowDrop?;
    private _maximizedTabSet?;
    private _activeTabSet?;
    private _borderRects;
    /**
     * 'private' constructor. Use the static method Model.fromJson(json) to create a model
     *  @hidden @internal
     */
    private constructor();
    _setChangeListener(listener: (() => void) | undefined): void;
    /**
     * Get the currently active tabset node
     */
    getActiveTabset(): TabSetNode | undefined;
    _setActiveTabset(tabsetNode: TabSetNode): void;
    /**
     * Get the currently maximized tabset node
     */
    getMaximizedTabset(): TabSetNode | undefined;
    _setMaximizedTabset(tabsetNode: TabSetNode): void;
    /**
     * Gets the root RowNode of the model
     * @returns {RowNode}
     */
    getRoot(): RowNode;
    /**
     * Gets the
     * @returns {BorderSet|*}
     */
    getBorderSet(): BorderSet;
    _getOuterInnerRects(): {
        inner: Rect;
        outer: Rect;
    };
    /**
     * Visits all the nodes in the model and calls the given function for each
     * @param fn a function that takes visited node and a integer level as parameters
     */
    visitNodes(fn: (node: Node, level: number) => void): void;
    /**
     * Gets a node by its id
     * @param id the id to find
     */
    getNodeById(id: string): Node;
    /**
     * Update the node tree by performing the given action,
     * Actions should be generated via static methods on the Actions class
     * @param action the action to perform
     */
    doAction(action: Action): void;
    _updateIdMap(): void;
    _adjustSplitSide(node: TabSetNode | RowNode, weight: number, pixels: number): void;
    /**
     * Converts the model to a json object
     * @returns {*} json object that represents this model
     */
    toJson(): any;
    /**
     * Loads the model from the given json object
     * @param json the json model to load
     * @returns {Model} a new Model object
     */
    static fromJson(json: any): Model;
    getSplitterSize(): number;
    isEnableEdgeDock(): boolean;
    _addNode(node: Node): void;
    _layout(rect: Rect): Rect;
    _findDropTargetNode(dragNode: Node & IDraggable, x: number, y: number): DropInfo | undefined;
    _tidy(): void;
    _updateAttrs(json: any): void;
    _nextUniqueId(): string;
    _getAttribute(name: string): any;
    /**
     * Sets a function to allow/deny dropping a node
     * @param onAllowDrop function that takes the drag node and DropInfo and returns true if the drop is allowed
     */
    setOnAllowDrop(onAllowDrop: (dragNode: Node, dropInfo: DropInfo) => boolean): void;
    _getOnAllowDrop(): ((dragNode: Node, dropInfo: DropInfo) => boolean) | undefined;
    toString(): string;
    private static _createAttributeDefinitions;
}
export default Model;
