import * as React from "react";
import Action from "../model/Action";
import BorderNode from "../model/BorderNode";
import BorderSet from "../model/BorderSet";
import IDraggable from "../model/IDraggable";
import Model from "../model/Model";
import Node from "../model/Node";
import RowNode from "../model/RowNode";
import TabNode from "../model/TabNode";
import TabSetNode from "../model/TabSetNode";
import { JSMap } from "../Types";
export interface ILayoutProps {
    model: Model;
    factory: (node: TabNode) => React.ReactNode;
    onAction?: (action: Action) => void;
    onRenderTab?: (node: TabNode, renderValues: {
        leading: React.ReactNode;
        content: React.ReactNode;
    }) => void;
    onRenderTabSet?: (tabSetNode: TabSetNode | BorderNode, renderValues: {
        headerContent?: React.ReactNode;
        buttons: Array<React.ReactNode>;
    }) => void;
    onModelChange?: (model: Model) => void;
}
/**
 * A React component that hosts a multi-tabbed layout
 */
export declare class Layout extends React.Component<ILayoutProps, any> {
    selfRef?: HTMLDivElement;
    private model?;
    private rect;
    private centerRect?;
    private tabIds;
    private newTabJson;
    private firstMove;
    private dragNode?;
    private dragDiv?;
    private dragDivText;
    private dropInfo;
    private outlineDiv?;
    private edgeRightDiv?;
    private edgeBottomDiv?;
    private edgeLeftDiv?;
    private edgeTopDiv?;
    private fnNewNodeDropped?;
    constructor(props: ILayoutProps);
    onModelChange(): void;
    doAction(action: Action): void;
    componentWillReceiveProps(newProps: ILayoutProps): void;
    componentDidMount(): void;
    componentDidUpdate(): void;
    updateRect(): void;
    componentWillUnmount(): void;
    render(): JSX.Element;
    renderBorder(borderSet: BorderSet, borderComponents: Array<React.ReactNode>, tabComponents: JSMap<React.ReactNode>, splitterComponents: Array<React.ReactNode>): void;
    renderChildren(node: RowNode | TabSetNode, tabSetComponents: Array<React.ReactNode>, tabComponents: JSMap<React.ReactNode>, splitterComponents: Array<React.ReactNode>): void;
    /**
     * Adds a new tab to the given tabset
     * @param tabsetId the id of the tabset where the new tab will be added
     * @param json the json for the new tab node
     */
    addTabToTabSet(tabsetId: string, json: any): void;
    /**
     * Adds a new tab to the active tabset (if there is one)
     * @param json the json for the new tab node
     * @hidden @internal
     */
    addTabToActiveTabSet(json: any): void;
    /**
     * Adds a new tab by dragging a labeled panel to the drop location, dragging starts immediatelly
     * @param dragText the text to show on the drag panel
     * @param json the json for the new tab node
     * @param onDrop a callback to call when the drag is complete
     */
    addTabWithDragAndDrop(dragText: string, json: any, onDrop: () => void): void;
    /**
     * Adds a new tab by dragging a labeled panel to the drop location, dragging starts when you
     * mouse down on the panel
     *
     * @param dragText the text to show on the drag panel
     * @param json the json for the new tab node
     * @param onDrop a callback to call when the drag is complete
     */
    addTabWithDragAndDropIndirect(dragText: string, json: any, onDrop: () => void): void;
    onCancelAdd(): void;
    onCancelDrag(wasDragging: boolean): void;
    onDragDivMouseDown(event: Event): void;
    dragStart(event: Event | undefined, dragDivText: string, node: Node & IDraggable, allowDrag: boolean, onClick?: (event: Event) => void, onDoubleClick?: (event: Event) => void): void;
    onDragStart(event: React.FormEvent<Element>): boolean;
    onDragMove(event: React.MouseEvent<Element>): void;
    onDragEnd(event: React.FormEvent<Element>): void;
    showEdges(rootdiv: HTMLElement): void;
    hideEdges(rootdiv: HTMLElement): void;
    maximize(tabsetNode: TabSetNode): void;
    customizeTab(tabNode: TabNode, renderValues: {
        leading: React.ReactNode;
        content: React.ReactNode;
    }): void;
    customizeTabSet(tabSetNode: TabSetNode | BorderNode, renderValues: {
        headerContent?: React.ReactNode;
        buttons: Array<React.ReactNode>;
    }): void;
}
export default Layout;
